==> 0.9.0.tar.gz: lib/src/infra/app_viewmodel.dart <==
import 'dart:core';
import 'package:flutter/material.dart';
import 'package:modular_system_core/mscore.dart';

enum Environment {
  development,
  production,
}

enum FontSize {
  title,
  subtitle,
  data,
  label,
}

// primary == white           // primaryContainer == white
// onPrimary == black         // onPrimaryContainer == black

// secondary == blue
// onSecondary == light blue

// tertiary == orange
// onTerticary == white

enum FontType {
  onPrimary,
  onPrimaryContainer,
  onSecondary,
  onSecondaryContainer,
  onTertiary,
  onTertiaryContainer,
  onBackground,
  onError,
  onErrorContainer,
}

class FontSizeConfig {
  FontSizeConfig({
    required this.title,
    required this.subtitle,
    required this.data,
    required this.label,
  });

  double title;
  double subtitle;
  double data;
  double label;
}

class AppBottomSheet {
  AppBottomSheet({
    required this.child,
    this.shape = const ContinuousRectangleBorder(),
  });

  Widget child;
  ShapeBorder shape;

  factory AppBottomSheet.empitySquared() {
    return AppBottomSheet(
      child: Container(),
      shape: const RoundedRectangleBorder(),
    );
  }

  factory AppBottomSheet.empityRounded() {
    return AppBottomSheet(
      child: Container(),
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(80),
          topRight: Radius.circular(80),
        ),
      ),
    );
  }

  factory AppBottomSheet.squared(Widget child) {
    return AppBottomSheet(
      child: child,
      shape: const RoundedRectangleBorder(),
    );
  }

  factory AppBottomSheet.rounded(Widget child) {
    return AppBottomSheet(
      child: child,
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(80),
          topRight: Radius.circular(80),
        ),
      ),
    );
  }
}

// REFACTORING

class AppState extends ReactiveObject<AppState> {
  AuthenticationState authenticationState = AuthenticationState();
}

class AuthenticationState extends ReactiveObject<AuthenticationState> {
  bool isAuthenticated = false;
  String token = "";
  String refreshToken = "";
  String userId = "";
  String username = "";
  DateTime tokenExpiresAt = DateTime.now();
}

class AppEvents {
  SetEvent<ColorScheme> colorScheme = SetEvent();
  IoEvent<AppBottomSheet, Future<dynamic>> bottomSheet = IoEvent();
}

class AppConfig {
  AppConfig({
    required this.environment,
    required this.serverUri,
    required this.fontSize,
    required this.colorScheme,
    this.animationDuration = const Duration(milliseconds: 250),
  });

  Environment environment;
  String serverUri;
  FontSizeConfig fontSize;
  ColorScheme colorScheme;
  // optional with defaults
  Duration animationDuration;
}

class AppViewModel {
  AppViewModel(
    this.config,
  );

  AppConfig config;
  AppState coreState = AppState();
  AppEvents coreEvents = AppEvents();
  ColorScheme get colors {
    return config.colorScheme;
  }

  late IStyleProvider styleProvider = StyleProvider(config);
  late IErrorHandler errorHandler = ErrorHandler(rootContext, config, styleProvider);
  late IInformer infoHandler = Informer(rootContext, config, styleProvider);
  late ILoader loader = Loader(config);
  late BuildContext rootContext;

  void showBottomSheet(Widget widget) {}

  Future<bool> authenticateAsync(CredentialHandshake handshake) async {
    return false;
  }
}

// Interfaces

abstract class IStyleProvider {
  TextStyle auto([FontType type = FontType.onPrimary, FontSize size = FontSize.data]);
}

abstract class IErrorHandler {
  void handle(Object e);
  void handleError(Error e, [StackTrace? stackTrace]);
  void handleException(Exception e, [StackTrace? stackTrace]);
  void handleAppException(AppException e);
}

abstract class IInformer {
  void inform(String message);
}

abstract class ILoader {
  Widget getLoader();
}

class StyleProvider implements IStyleProvider {
  StyleProvider(this.config);

  AppConfig config;

  @override
  TextStyle auto([FontType type = FontType.onPrimary, FontSize size = FontSize.data]) {
    return TextStyle(
      color: getColorFrom(type),
      fontSize: getSizeFrom(size),
    );
  }

  Color getColorFrom(FontType type) {
    switch (type) {
      // primary color
      case FontType.onPrimary:
        return config.colorScheme.onPrimary;
      case FontType.onPrimaryContainer:
        return config.colorScheme.onPrimaryContainer;
      // secondary color
      case FontType.onSecondary:
        return config.colorScheme.onSecondary;
      case FontType.onSecondaryContainer:
        return config.colorScheme.onSecondaryContainer;
      // tertiary color
      case FontType.onTertiary:
        return config.colorScheme.onTertiary;
      case FontType.onTertiaryContainer:
        return config.colorScheme.onTertiaryContainer;
      // background
      case FontType.onBackground:
        return config.colorScheme.onBackground;
      // error
      case FontType.onError:
        return config.colorScheme.onError;
      case FontType.onErrorContainer:
        return config.colorScheme.onErrorContainer;

      default:
        return config.colorScheme.onBackground;
    }
  }

  double getSizeFrom(FontSize size) {
    switch (size) {
      case FontSize.title:
        return config.fontSize.title;
      case FontSize.subtitle:
        return config.fontSize.subtitle;
      case FontSize.label:
        return config.fontSize.label;
      case FontSize.data:
        return config.fontSize.data;
      default:
        return config.fontSize.data;
    }
  }
}

class ErrorHandler implements IErrorHandler {
  ErrorHandler(
    this.context,
    this.config,
    this.styleProvider,
  );

  BuildContext context;
  AppConfig config;
  IStyleProvider styleProvider;

  @override
  void handle(Object e) {
    if (e is Error) {
      handleError(e);
    } else if (e is Exception) {
      handleException(e);
    } else if (e is AppException) {
      handleAppException(e);
    }
  }

  @override
  void handleError(Error e, [StackTrace? stackTrace]) {
    var message = e.toAppException(stackTrace).toString(config.environment);
    showErrorSnackbar(message);
  }

  @override
  void handleException(Exception e, [StackTrace? stackTrace]) {
    var message = e.toAppException(stackTrace).toString(config.environment);
    showErrorSnackbar(message);
  }

  @override
  void handleAppException(AppException e) {
    var message = e.toString(config.environment);
    showErrorSnackbar(message);
  }

  void showErrorSnackbar(String message) {
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        duration: const Duration(
          milliseconds: 8000,
        ),
        backgroundColor: config.colorScheme.errorContainer,
        behavior: SnackBarBehavior.fixed,
        padding: EdgeInsets.symmetric(
          horizontal: config.fontSize.data * 3,
          vertical: config.fontSize.data * 2,
        ),
        content: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Icon(
              Icons.warning,
              color: config.colorScheme.onErrorContainer,
              size: config.fontSize.data,
            ),
            Flexible(
              fit: FlexFit.tight,
              child: Container(
                margin: EdgeInsets.symmetric(
                  horizontal: config.fontSize.data * 2,
                ),
                child: SelectableText(
                  message,
                  minLines: 1,
                  maxLines: 5,
                  style: styleProvider.auto(FontType.onErrorContainer),
                ),
              ),
            ),
            IconButton(
              icon: Icon(
                Icons.close,
                color: config.colorScheme.onErrorContainer,
                size: config.fontSize.data,
              ),
              onPressed: () {
                ScaffoldMessenger.of(context).hideCurrentSnackBar();
              },
            ),
          ],
        ),
      ),
    );
  }
}

class Informer implements IInformer {
  Informer(
    this.context,
    this.config,
    this.styleProvider,
  );

  BuildContext context;
  AppConfig config;
  IStyleProvider styleProvider;

  @override
  void inform(String message) {
    showInfoSnackbar(message);
  }

  void showInfoSnackbar(String message) {
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        duration: const Duration(
          milliseconds: 8000,
        ),
        backgroundColor: config.colorScheme.tertiaryContainer,
        behavior: SnackBarBehavior.fixed,
        padding: EdgeInsets.symmetric(
          horizontal: config.fontSize.data * 3,
          vertical: config.fontSize.data * 2,
        ),
        content: Row(
          children: [
            Icon(
              Icons.info,
              color: config.colorScheme.onTertiaryContainer,
              size: config.fontSize.data,
            ),
            Flexible(
              fit: FlexFit.tight,
              child: Container(
                margin: const EdgeInsets.symmetric(horizontal: 20),
                child: Text(
                  message,
                  maxLines: 5,
                  overflow: TextOverflow.ellipsis,
                  style: styleProvider.auto(FontType.onTertiaryContainer),
                ),
              ),
            ),
            IconButton(
              icon: Icon(
                Icons.close,
                color: config.colorScheme.onTertiaryContainer,
                size: config.fontSize.data,
              ),
              onPressed: () {
                ScaffoldMessenger.of(context).hideCurrentSnackBar();
              },
            ),
          ],
        ),
      ),
    );
  }
}

class Loader implements ILoader {
  Loader(this.config);

  AppConfig config;

  @override
  Widget getLoader() {
    return CircularProgressIndicator(
      color: config.colorScheme.onBackground,
    );
  }
}

==> 0.9.3.tar.gz: lib/src/infra/app_viewmodel.dart <==
import 'dart:core';
import 'package:flutter/material.dart';
import 'package:modular_system_core/mscore.dart';

enum Environment {
  development,
  production,
}

enum FontType {
  onPrimary,
  onPrimaryContainer,
  onSecondary,
  onSecondaryContainer,
  onTertiary,
  onTertiaryContainer,
  onBackground,
  onError,
  onErrorContainer,
}

enum FontSize {
  title,
  subtitle,
  data,
  label,
}

class FontSizeScheme {
  FontSizeScheme({
    required this.title,
    required this.subtitle,
    required this.data,
    required this.label,
  });

  double title;
  double subtitle;
  double data;
  double label;
}

class AppBottomSheet {
  AppBottomSheet({
    required this.child,
    this.shape = const ContinuousRectangleBorder(),
  });

  Widget child;
  ShapeBorder shape;

  factory AppBottomSheet.empitySquared() {
    return AppBottomSheet(
      child: Container(),
      shape: const RoundedRectangleBorder(),
    );
  }

  factory AppBottomSheet.empityRounded() {
    return AppBottomSheet(
      child: Container(),
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(80),
          topRight: Radius.circular(80),
        ),
      ),
    );
  }

  factory AppBottomSheet.squared(Widget child) {
    return AppBottomSheet(
      child: child,
      shape: const RoundedRectangleBorder(),
    );
  }

  factory AppBottomSheet.rounded(Widget child) {
    return AppBottomSheet(
      child: child,
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(80),
          topRight: Radius.circular(80),
        ),
      ),
    );
  }
}

// REFACTORING

class AppState extends ReactiveObject<AppState> {
  AuthenticationState authenticationState = AuthenticationState();
}

class AuthenticationState extends ReactiveObject<AuthenticationState> {
  bool isAuthenticated = false;
  String token = "";
  String refreshToken = "";
  String userId = "";
  String username = "";
  DateTime tokenExpiresAt = DateTime.now();
}

class AppEvents {
  SetEvent<ColorScheme> colorScheme = SetEvent();
  IoEvent<AppBottomSheet, Future<dynamic>> bottomSheet = IoEvent();
}

class AppConfig {
  AppConfig({
    required this.environment,
    required this.serverUri,
    required this.fontSize,
    required this.colorScheme,
    this.animationDuration = const Duration(milliseconds: 250),
  });

  Environment environment;
  String serverUri;
  FontSizeScheme fontSize;
  ColorScheme colorScheme;
  // optional with defaults
  Duration animationDuration;
}

class AppViewModel {
  AppViewModel(
    this.config,
  );

  AppConfig config;
  AppState coreState = AppState();
  AppEvents coreEvents = AppEvents();
  ColorScheme get colors {
    return config.colorScheme;
  }

  FontSizeScheme get fontSize {
    return config.fontSize;
  }

  late IStyleProvider styleProvider = StyleProvider(config);
  late IErrorHandler errorHandler = ErrorHandler(rootContext, config, styleProvider);
  late IInformer informer = Informer(rootContext, config, styleProvider);
  late ILoader loader = Loader(config);
  late BuildContext rootContext;

  void showBottomSheet(Widget widget) {}

  Future<bool> authenticateAsync(CredentialHandshake handshake) async {
    return false;
  }
}

// Interfaces

abstract class IStyleProvider {
  TextStyle auto([FontType type = FontType.onPrimary, FontSize size = FontSize.data]);
}

abstract class IErrorHandler {
  void handle(Object e);
  void handleError(Error e, [StackTrace? stackTrace]);
  void handleException(Exception e, [StackTrace? stackTrace]);
  void handleAppException(AppException e);
}

abstract class IInformer {
  void inform(String message);
}

abstract class ILoader {
  Widget getLoader([Color color]);
}

class StyleProvider implements IStyleProvider {
  StyleProvider(this.config);

  AppConfig config;

  @override
  TextStyle auto([FontType type = FontType.onPrimary, FontSize size = FontSize.data]) {
    return TextStyle(
      color: getColorFrom(type),
      fontSize: getSizeFrom(size),
    );
  }

  Color getColorFrom(FontType type) {
    switch (type) {
      // primary color
      case FontType.onPrimary:
        return config.colorScheme.onPrimary;
      case FontType.onPrimaryContainer:
        return config.colorScheme.onPrimaryContainer;
      // secondary color
      case FontType.onSecondary:
        return config.colorScheme.onSecondary;
      case FontType.onSecondaryContainer:
        return config.colorScheme.onSecondaryContainer;
      // tertiary color
      case FontType.onTertiary:
        return config.colorScheme.onTertiary;
      case FontType.onTertiaryContainer:
        return config.colorScheme.onTertiaryContainer;
      // background
      case FontType.onBackground:
        return config.colorScheme.onBackground;
      // error
      case FontType.onError:
        return config.colorScheme.onError;
      case FontType.onErrorContainer:
        return config.colorScheme.onErrorContainer;

      default:
        return config.colorScheme.onBackground;
    }
  }

  double getSizeFrom(FontSize size) {
    switch (size) {
      case FontSize.title:
        return config.fontSize.title;
      case FontSize.subtitle:
        return config.fontSize.subtitle;
      case FontSize.label:
        return config.fontSize.label;
      case FontSize.data:
        return config.fontSize.data;
      default:
        return config.fontSize.data;
    }
  }
}

class ErrorHandler implements IErrorHandler {
  ErrorHandler(
    this.context,
    this.config,
    this.styleProvider,
  );

  BuildContext context;
  AppConfig config;
  IStyleProvider styleProvider;

  @override
  void handle(Object e) {
    if (e is Error) {
      handleError(e);
    } else if (e is Exception) {
      handleException(e);
    } else if (e is AppException) {
      handleAppException(e);
    }
  }

  @override
  void handleError(Error e, [StackTrace? stackTrace]) {
    var message = e.toAppException(stackTrace).toString(config.environment);
    showErrorSnackbar(message);
  }

  @override
  void handleException(Exception e, [StackTrace? stackTrace]) {
    var message = e.toAppException(stackTrace).toString(config.environment);
    showErrorSnackbar(message);
  }

  @override
  void handleAppException(AppException e) {
    var message = e.toString(config.environment);
    showErrorSnackbar(message);
  }

  void showErrorSnackbar(String message) {
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        duration: const Duration(
          milliseconds: 8000,
        ),
        backgroundColor: config.colorScheme.errorContainer,
        behavior: SnackBarBehavior.fixed,
        padding: EdgeInsets.symmetric(
          horizontal: config.fontSize.data * 3,
          vertical: config.fontSize.data * 2,
        ),
        content: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Icon(
              Icons.warning,
              color: config.colorScheme.onErrorContainer,
              size: config.fontSize.data,
            ),
            Flexible(
              fit: FlexFit.tight,
              child: Container(
                margin: EdgeInsets.symmetric(
                  horizontal: config.fontSize.data * 2,
                ),
                child: SelectableText(
                  message,
                  minLines: 1,
                  maxLines: 5,
                  style: styleProvider.auto(FontType.onErrorContainer),
                ),
              ),
            ),
            IconButton(
              icon: Icon(
                Icons.close,
                color: config.colorScheme.onErrorContainer,
                size: config.fontSize.data,
              ),
              onPressed: () {
                ScaffoldMessenger.of(context).hideCurrentSnackBar();
              },
            ),
          ],
        ),
      ),
    );
  }
}

class Informer implements IInformer {
  Informer(
    this.context,
    this.config,
    this.styleProvider,
  );

  BuildContext context;
  AppConfig config;
  IStyleProvider styleProvider;

  @override
  void inform(String message) {
    showInfoSnackbar(message);
  }

  void showInfoSnackbar(String message) {
    ScaffoldMessenger.of(context).showSnackBar(
      SnackBar(
        duration: const Duration(
          milliseconds: 8000,
        ),
        backgroundColor: config.colorScheme.tertiaryContainer,
        behavior: SnackBarBehavior.fixed,
        padding: EdgeInsets.symmetric(
          horizontal: config.fontSize.data * 3,
          vertical: config.fontSize.data * 2,
        ),
        content: Row(
          children: [
            Icon(
              Icons.info,
              color: config.colorScheme.onTertiaryContainer,
              size: config.fontSize.data,
            ),
            Flexible(
              fit: FlexFit.tight,
              child: Container(
                margin: const EdgeInsets.symmetric(horizontal: 20),
                child: Text(
                  message,
                  maxLines: 5,
                  overflow: TextOverflow.ellipsis,
                  style: styleProvider.auto(FontType.onTertiaryContainer),
                ),
              ),
            ),
            IconButton(
              icon: Icon(
                Icons.close,
                color: config.colorScheme.onTertiaryContainer,
                size: config.fontSize.data,
              ),
              onPressed: () {
                ScaffoldMessenger.of(context).hideCurrentSnackBar();
              },
            ),
          ],
        ),
      ),
    );
  }
}

class Loader implements ILoader {
  Loader(this.config);

  final AppConfig config;

  @override
  Widget getLoader([Color color = Colors.black]) {
    return CircularProgressIndicator(
      color: color,
    );
  }
}
